<?php
/**
 *  @package Invoicing
 *  @copyright Copyright (c)2015 Nicholas K. Dionysopoulos, Juloa
 *  @license GNU General Public License version 3, or later
 *  Code adapted from Akeeba Subscription
 */

defined('_JEXEC') or die();

include_once(JPATH_ADMINISTRATOR.'/components/com_invoicing/helpers/cparams.php');
include_once(JPATH_ADMINISTRATOR.'/components/com_invoicing/helpers/format.php');
include_once(JPATH_ADMINISTRATOR.'/components/com_invoicing/helpers/pdf.php');
include_once(JPATH_ADMINISTRATOR.'/components/com_invoicing/assets/view.pdf.php');
jimport( 'joomla.filesystem.file' );

class InvoicingHelperEmail
{
	/**
	 * Gets the email keys currently known to the component
	 *
	 * @param   integer  $style  0 = raw sections list, 1 = grouped list options, 2 = key/description array
	 *
	 * @return  array|string
	 */
	public static function getEmailTypes()
	{
		static $listTypeEmails = null;
        
    return $listTypeEmails;
	}

	/**
	 * Loads an email template from the database or, if it doesn't exist, from
	 * the language file.
	 *
	 * @param   string   $key    The language key, in the form PLG_LOCATION_PLUGINNAME_TYPE
	 *
	 * @return  array  isHTML: If it's HTML override from the db; text: The unprocessed translation string
	 */
	private static function loadEmailTemplate($key, $user = null)
	{
		if (is_null($user))
		{
			$user = JFactory::getUser();
		}
        
		// Initialise
		$templateText = '';
		$subject = '';
		$isHTML = false;

		// Look for desired languages
		$jLang = JFactory::getLanguage();
		$userLang = $user->getParam('language','');
		$languages = array(
			$userLang, $jLang->getTag(), $jLang->getDefault(), 'en-GB', '*'
		);

		// Look for an override in the database
		$db = JFactory::getDbo();
		$query = $db->getQuery(true)
			->select('*')
			->from($db->qn('#__invoicing_emails'))
			->where($db->qn('key').'='.$db->q($key))
			->where($db->qn('enabled').'='.$db->q(1))
		;
		$db->setQuery($query);
		$allTemplates = $db->loadObjectList();
        
		if(!empty($allTemplates))
		{
			// Pass 1 - Give match scores to each template
			$preferredIndex = null;
			$preferredScore = 0;
			foreach($allTemplates as $idx => $template)
			{
				// Get the language and level of this template
				$myLang = $template->language;
				//$myType = $template->usersubs_type_id;

				// Make sure the language matches one of our desired languages, otherwise skip it
				$langPos = array_search($myLang, $languages);
				if ($langPos === false)
				{
					continue;
				}
				$langScore = (5 - $langPos);

				// Calculate the score. If it's winning, use it
				$score = $langScore;
				if ($score > $preferredScore)
				{
					$loadLanguage = $myLang;
					$subject = $template->subject;
					$templateText = $template->body;
					$preferredScore = $score;

					$isHTML = true;
				}
			}
		}

		return array($isHTML, $subject, $templateText);
	}

	/**
	 * Creates a PHPMailer instance
	 *
	 * @param   boolean  $isHTML
	 *
	 * @return  PHPMailer  A mailer instance
	 */
	private static function &getMailer($isHTML = true)
	{
		$mailer = clone JFactory::getMailer();

		$mailer->IsHTML($isHTML);
		// Required in order not to get broken characters
		$mailer->CharSet = 'UTF-8';

		return $mailer;
	}

	/**
	 * Creates a mailer instance, preloads its subject and body with your email
	 * data based on the key and extra substitution parameters and waits for
	 * you to send a recipient and send the email.
	 *
	 * @param   object  $item     The subscription record against which the email is sent
	 * @param   string  $key     The email key, in the form PLG_LOCATION_PLUGINNAME_TYPE
	 * @param   array   $extras  Any optional substitution strings you want to introduce
	 *
	 * @return  boolean|PHPMailer False if something bad happened, the PHPMailer instance in any other case
	 */
	public static function getPreloadedMailer($item, $key, array $extras = array())
	{
		// Load the template
		list($isHTML, $subject, $templateText) = self::loadEmailTemplate($key, JFactory::getUser($item->user_id));
		if (empty($subject))
		{
			return false;
		}

		$templateText = InvoicingHelperFormat::replaceTags($templateText, $item, $extras);
		$subject = InvoicingHelperFormat::replaceTags($subject, $item, $extras);

		// Get the mailer
		$mailer = self::getMailer($isHTML);
		$mailer->setSubject($subject);

		// Include inline images
		$pattern = '/(src)=\"([^"]*)\"/i';
		$number_of_matches = preg_match_all($pattern, $templateText, $matches, PREG_OFFSET_CAPTURE);
		if($number_of_matches > 0) {
			$substitutions = $matches[2];
			$last_position = 0;
			$temp = '';

			// Loop all URLs
			$imgidx = 0;
			$imageSubs = array();
			foreach($substitutions as &$entry)
			{
				// Copy unchanged part, if it exists
				if($entry[1] > 0)
					$temp .= substr($templateText, $last_position, $entry[1]-$last_position);
				// Examine the current URL
				$url = $entry[0];
				if( (substr($url,0,7) == 'http://') || (substr($url,0,8) == 'https://') ) {
					// External link, skip
					$temp .= $url;
				} else {
					$ext = strtolower(JFile::getExt($url));
					if(!JFile::exists($url)) {
						// Relative path, make absolute
						$url = dirname($template).'/'.ltrim($url,'/');
					}
					if( !JFile::exists($url) || !in_array($ext, array('jpg','png','gif')) ) {
						// Not an image or inexistent file
						$temp .= $url;
					} else {
						// Image found, substitute
						if(!array_key_exists($url, $imageSubs)) {
							// First time I see this image, add as embedded image and push to
							// $imageSubs array.
							$imgidx++;
							$mailer->AddEmbeddedImage($url, 'img'.$imgidx, basename($url));
							$imageSubs[$url] = $imgidx;
						}
						// Do the substitution of the image
						$temp .= 'cid:img'.$imageSubs[$url];
					}
				}

				// Calculate next starting offset
				$last_position = $entry[1] + strlen($entry[0]);
			}
			// Do we have any remaining part of the string we have to copy?
			if($last_position < strlen($templateText))
				$temp .= substr($templateText, $last_position);
			// Replace content with the processed one
			$templateText = $temp;
		}
        
		$mailer->setBody($templateText);

		return $mailer;
	}
    
    /**
	 * Sends out the email to the owner of the subscription.
	 *
	 * @param $item USersubsSubscription The subscription row object
	 * @param $type string The type of the email to send (generic, new,)
	 */
	public static function sendEmail($item, $type = '')
	{		
		// Get a preloaded mailer
		$key = $type;
		$mailer = self::getPreloadedMailer($item, $key);

		if ($mailer === false)
		{
			return false;
		}
        
    $mailer->addRecipient($user->email);
      
		$result = $mailer->Send();
		$mailer = null;

		return $result;
	}
    
  public static function getPDFInvoice($invoice) {
		$filenameText = JText::_('INVOICING_INVOICE_FILENAME');
		$invoiceNumber = InvoicingHelperFormat::formatInvoiceNumber($invoice);
		$filename = sprintf($filenameText,$invoiceNumber);
		$filename = str_replace(array("/","."," "),"_",$filename);
		$filename =  JPATH_SITE.'/tmp/'.$filename.'.pdf';
		$data = InvoicingHelperFormat::formatInvoiceHTML($invoice);
		InvoicingHelperPDF::createPDF($data,$filename);
		return $filename;
	}

	public static function getPDFOrder($invoice) {
		$filenameText = JText::_('INVOICING_ORDER_FILENAME');
		$invoiceNumber = InvoicingHelperFormat::formatOrderNumber($invoice);
		$filename = sprintf($filenameText,$invoiceNumber);
		$filename = str_replace(array("/","."," "),"_",$filename);
		$filename =  JPATH_SITE.'/tmp/'.$filename.'.pdf';
		$data = InvoicingHelperFormat::formatOrderHTML($invoice);
		InvoicingHelperPDF::createPDF($data,$filename);
		return $filename;
	}
	
	public static function getPDFQuote($quote) {
		$filenameText = JText::_('INVOICING_QUOTE_FILENAME');
		$invoiceNumber = InvoicingHelperFormat::formatOrderNumber($invoice);
		$filename = sprintf($filenameText,$invoiceNumber);
		$filename = str_replace(array("/","."," "),"_",$filename);
		$filename =  JPATH_SITE.'/tmp/'.$filename.'.pdf';
		$data = InvoicingHelperFormat::formatQuoteHTML($quote);
		InvoicingHelperPDF::createPDF($data,$filename);
		return $filename;
	}
}